""""
Utility Functions for evaluating the status and structure of python code
"""

import argparse
import difflib
import os
import tempfile
from io import StringIO

import black
from nbconvert import PythonExporter
from pyflakes.api import checkPath
from pyflakes.reporter import Reporter
from pylint.lint import Run
from pylint.reporters.text import TextReporter


def convert_temp_names_to_originals(
    errors_and_warnings: list, original_path: str
):
    """In a list of errors ["<path>: <error>"], converts path to original_path

    When Jupyter Notebooks are converted to python temp files and run throughj
    pyflakes, the reported errors are for the temp files. This function converts
    them back to be more descriptive

    Args:
        errors_and_warnings: list of pyflakes errors and warnings of the format
            ["<path>: <error/warning>"]
        original_path: name of original jupyter notebook
    Returns: list with format ["<original_path>: <error/warning>"]
    """
    # assuming format of <path> is <path>:<line no>:<column no>:, we want first colon
    return [
        original_path + path_and_message[path_and_message.find(":") :]
        for path_and_message in errors_and_warnings
    ]


def pyflakes_notebook(path_to_notebook):
    """
    Run pyflakes on a Jupyter notebook.
    :param path_to_notebook: path to the notebook file.
    :return: list of warnings and errors from pyflakes.
    """
    # Convert notebook to Python script
    exporter = PythonExporter()
    script, _ = exporter.from_filename(path_to_notebook)

    with tempfile.NamedTemporaryFile(
        mode="w", delete=False, suffix=".py"
    ) as temp:
        temp_name = temp.name
        temp.write(script)

    errors_and_warnings = pyflakes_python_file(temp_name)
    reformatted_errors_and_warnings = convert_temp_names_to_originals(
        errors_and_warnings,
        path_to_notebook,
    )

    # Delete temporary file
    os.remove(temp_name)
    return reformatted_errors_and_warnings


def pyflakes_python_file(file_path):
    """
    Run a python file through pyflakes. returns the number of warnings raised.
    """
    # Prepare StringIO object for capturing pyflakes output
    error_stream = StringIO()
    warning_stream = StringIO()

    reporter = Reporter(warning_stream, error_stream)

    # Run pyflakes and capture output
    checkPath(file_path, reporter=reporter)

    # Get errors and warnings
    errors = [
        ":".join([file_path] + x.split(":")[1:])
        for x in error_stream.getvalue().splitlines()
    ]

    warnings = [
        ":".join([file_path] + x.split(":")[1:])
        for x in warning_stream.getvalue().splitlines()
    ]

    return errors + warnings


def parse_arguments():
    """
    Parse command line arguments.
    """
    parser = argparse.ArgumentParser(description="Process Cod Repository.")
    parser.add_argument(
        "dir_path", type=str, help="Directory path to start the search from"
    )
    return parser.parse_args()


def get_pylint_warnings(filepath):
    """
    This function (generated by chatgpt) runs pylint on files and r
    eturns all warnings
    """
    pylint_output = StringIO()
    reporter = TextReporter(pylint_output)
    Run([filepath], reporter=reporter, do_exit=False)
    pylint_output.seek(0)
    output_lines = pylint_output.readlines()
    # warnings = [line for line in output_lines if 'warning' in line.lower()]
    warnings = [line.replace("\n", "") for line in output_lines]
    return warnings


def black_python_file(file_path):
    try:
        # Read the content of the file
        with open(file_path, "r", encoding="utf-8") as f:
            content = f.read()

        # Use Black's API to format the code
        formatted_content = black.format_file_contents(
            content,
            fast=False,
            mode=black.FileMode(),
        )

        # Compare original and formatted content
        diff = difflib.unified_diff(
            content.splitlines(),
            formatted_content.splitlines(),
            fromfile="original",
            tofile="formatted",
            lineterm="",
        )

        diff_list = list(diff)

        # diff_str = "\n".join(diff)
        return diff_list if len(diff_list) > 0 else None

    except black.NothingChanged:
        return None
    except Exception as e:
        return f"Unexpected error: {e}"
